/*-
 * Copyright (C)2007 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C)2007 @BABOLO http://www.babolo.ru/\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: collkill.c,v 1.2 2008/01/22 07:58:49 babolo Exp $";
#endif /* not lint */

/*    pid    */
#include <sys/types.h>
#include <sysexits.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <stdio.h>
#include <mife.h>
#include <err.h>
#include <babolo/BLINflag.h>

#define COLLKILL_PROC 0x01
#define COLLKILL_KILL 0x02
#define COLLKILL_AMAX 0x04

int
main(int argc, char **argv) {
    pid_t pid = 0;
    int sig = 0, c, a;
    u_int32_t flags;
    mife_descriptor *md;
    char *procmd, *q, *p, *n;
    ssize_t argl;

    flags = BLIN_VER1;
    while((c = getopt(argc, argv, "k:mp:qv")) > 0) {
        switch (c) {
        case 'k':
            sig = strtol(optarg, NULL, 0);
            flags |= COLLKILL_KILL;
            break;
        case 'm':
            flags |= COLLKILL_AMAX;
            break;
        case 'p':
            pid = strtol(optarg, NULL, 0);
            flags |= COLLKILL_PROC;
            break;
        case 'q':
            flags &= ~BLIN_VERMASK;
            break;
        case 'v':
            BLIN_VERBOSE(flags);
            break;
    }   }
    argc -= optind;
    argv += optind;
    if  (~flags & COLLKILL_PROC) {
        ifBLIN_QV1(flags) warnx("No pid");
        exit(EX_USAGE);
    }
    if  ((asprintf(&procmd, "/proc/%d/cmdline", pid)) < 0) {
        ifBLIN_QV1(flags) warn("No mem");
        exit(EX_OSERR);
    }
    if  (!(md = mife_open(MIFE_PIPE, procmd))) {
        ifBLIN_QV1(flags) warn("mife open");
        exit(EX_IOERR);
    }
    if  ((argl = mife_read(md, 0, 0)) < 0) {
        ifBLIN_QV1(flags) warn("mife read");
        exit(EX_IOERR);
    }
    if  (!(n = p = mife_get(md, 0))) {
        ifBLIN_QV1(flags) warn("mife get");
        exit(EX_IOERR);
    }
    a = 0;
    q = argv[a];
    ifBLIN_QV3(flags) fprintf(stderr, "[%d]=%s~\n", a, q);
    for (c = 0; a < argc;) {
        ifBLIN_QV4(flags) fprintf(stderr, "%02X'%c' <?> %02X'%c'", *p, *p, *q, *q);
        c = *(p++) - *q;
        ifBLIN_QV4(flags) fprintf(stderr, "%s\n", c ? "<>" : "==");
        if  (c) {
            ifBLIN_QV2(flags) printf("Difference in arg %d, char %d\n", a, q - argv[a]);
            exit(1);
        } else {
            if  (!*(q++)) {
                q = argv[++a];
                ifBLIN_QV3(flags) fprintf(stderr, "[%d]=%s~\n", a, q);
    }   }   }
    if  ((~flags & COLLKILL_AMAX) && (argl != (p - n))) {
        ifBLIN_QV2(flags) printf("Difference arg %d\n", a);
        exit(1);
    }
    if  (flags & COLLKILL_KILL) {
        ifBLIN_QV2(flags) printf("PID=%d SIG=%d\n", pid, sig);
        if  ((kill(pid, sig)) < 0) {
            ifBLIN_QV1(flags) warn("kill");
            exit(EX_UNAVAILABLE);
    }   }
    exit(EX_OK);
}
